const { src, dest, parallel, watch } = require('gulp');
const $ = require('./modules.js');
const uglify = $.composer($.uglifyes, $.composer);

const path = {
  src: './src',
  dist: './dist',
  distAssets: './dist/assets',
};

function html() {
  return src([`${path.src}/ejs/*.ejs`, `!${path.src}/ejs/**/_*.ejs`])
    .pipe(
        $.plumber({
          errorHandler: $.notify.onError('Error: <%= error.message %>'),
        })
      )
    .pipe($.ejs())
    .pipe($.rename({ extname: ".html" }))
    .pipe(dest(path.dist))
    .pipe(
      $.browserSync.reload({
        stream: true,
        once: true,
      })
    );
}

function css() {
  return src(`${path.src}/scss/*.scss`)
    .pipe($.sourcemaps.init())
    .pipe($.sass.sync({outputStyle: 'expanded'}))
    .on('error', $.sass.logError)
    // .pipe($.sass.sync({outputStyle: 'expanded'}).on('error', gulpsass.logError))
    .pipe($.sassGlob())
    .pipe($.autoprefixer())
    .pipe($.sourcemaps.write(`/`))
    .pipe(dest(`${path.distAssets}/css`))
    .pipe(
      $.rename({
        suffix: '.min',
      })
    )
    .pipe($.minifyCSS())
    .pipe(dest(`${path.distAssets}/css`))
    .pipe(
      $.browserSync.reload({
        stream: true,
        once: true,
      })
    );
}

function js() {
  return src(`${path.src}/js/**/*.js`, { sourcemaps: true })
    .pipe(
      $.plumber({
        errorHandler: $.notify.onError('Error: <%= error.message %>'),
      })
    )
    .pipe(uglify({ output: { comments: /^!/ } }))
    .pipe(
      $.concat('main.min.js', {
        newLine: '\n',
      })
    )
    .pipe(dest(`${path.distAssets}/js`, { sourcemaps: true }))
    .pipe(
      $.browserSync.reload({
        stream: true,
        once: true,
      })
    );
}

function images() {
  return src(`${path.src}/images/**/**`)
    .pipe($.changed(`${path.distAssets}/images/`))
    .pipe(
      $.imagemin({
        optimizationLevel: 3,
      })
    )
    .pipe(dest(`${path.distAssets}/images/`));
}

function bs() {
  $.browserSync.init({
    port: 3030,
    server: {
      baseDir: path.dist,
      index: 'index.html'
    },
    open: 'external',// IPで開く
  });
}

exports.html = html;
exports.css = css;
exports.js = js;
exports.images = images;
exports.bs = bs;

exports.default = parallel([html, css, js, images, bs], () => {
  watch(`${path.src}/ejs/**`, html);
  watch(`${path.src}/scss/**`, css);
  watch(`${path.src}/js/**`, js);
  watch(`${path.src}/images/**`, images);
});
